@@ -1,3 +1,20 @@
+# Licensed to the Apache Software Foundation (ASF) under one
+# or more contributor license agreements.  See the NOTICE file
+# distributed with this work for additional information
+# regarding copyright ownership.  The ASF licenses this file
+# to you under the Apache License, Version 2.0 (the
+# "License"); you may not use this file except in compliance
+# with the License.  You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+
 ## TODO
 ## this class is here because of https://github.com/gfx/p5-Mouse/pull/67
 ## once 2.4.7 version of Mouse in Ubuntu for affected Perl version
@@ -130,7 +147,7 @@ func _update_params(
             # faked an index here, to make optimizer create diff
             # state for the same index but on diff devs, TODO(mli)
             # use a better solution later
-            &{$updater}($index*$num_device+$k, $g, $w);
+            $updater->($index*$num_device+$k, $g, $w);
         }, $arg_list, $grad_list);
     }, $param_arrays, $grad_arrays);
 }
@@ -173,7 +190,7 @@ has 'state_names'       => (is => 'rw', isa => 'Maybe[ArrayRef[Str]]');
 has 'logger'            => (is => 'ro', default => sub { AI::MXNet::Logging->get_logger });
 has '_p'                => (is => 'rw', init_arg => undef);
 has 'context'           => (
-    is => 'ro', 
+    is => 'ro',
     isa => 'AI::MXNet::Context|ArrayRef[AI::MXNet::Context]',
     default => sub { AI::MXNet::Context->cpu }
 );
@@ -440,13 +457,13 @@ method init_params(
                     }
                     if(defined $initializer)
                     {
-                        &{$initializer}($name, $arr);
+                        $initializer->($name, $arr);
                     }
                 }
             }
             else
             {
-                &{$initializer}($name, $arr) if defined $initializer;
+                $initializer->($name, $arr) if defined $initializer;
             }
     };
     my $attrs = $self->_symbol->attr_dict;
@@ -944,4 +961,4 @@ method _kvstore()
     $self->_p->_kvstore;
 }
 
-1;
\ No newline at end of file
+1;
