@@ -1,3 +1,20 @@
+# Licensed to the Apache Software Foundation (ASF) under one
+# or more contributor license agreements.  See the NOTICE file
+# distributed with this work for additional information
+# regarding copyright ownership.  The ASF licenses this file
+# to you under the Apache License, Version 2.0 (the
+# "License"); you may not use this file except in compliance
+# with the License.  You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+
 package AI::MXNet::TestUtils;
 use strict;
 use warnings;
@@ -9,7 +26,7 @@ use Exporter;
 use base qw(Exporter);
 @AI::MXNet::TestUtils::EXPORT_OK = qw(same reldiff almost_equal GetMNIST_ubyte
                                       GetCifar10 pdl_maximum pdl_minimum mlp2 conv
-                                      check_consistency zip assert enumerate same_array dies_like);
+                                      check_consistency zip assert enumerate same_array dies_like allclose);
 use constant default_numerical_threshold => 1e-6;
 =head1 NAME
 
@@ -30,6 +47,21 @@ func same(PDL $a, PDL $b)
     return ($a != $b)->sum == 0;
 }
 
+=head2 allclose
+
+    Test if all elements of two pdl arrays are almost equal
+
+    Parameters
+    ----------
+    a : pdl
+    b : pdl
+=cut
+
+func allclose(PDL $a, PDL $b)
+{
+    return (($a - $b)->abs <= default_numerical_threshold)->all;
+}
+
 =head2 reldiff
 
     Calculate the relative difference between two input arrays
@@ -399,4 +431,4 @@ func dies_like($code, $regexp)
     }
 }
 
-1;
\ No newline at end of file
+1;
